﻿package object3ds.factoryObject3d
{
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.events.Event;
	
	import org.papervision3d.objects.parsers.DAE;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.core.animation.clip.AnimationClip3D;
	import org.papervision3d.events.FileLoadEvent;

	/**
	 * Colladaデータとテクスチャを読み込んでDAEオブジェクトとして利用できる状態にします。
	 * @author hagiwara
	 */
	public class FactoryObject3d extends DAE
	{
		public static const PREPARE_CREATE_OBJECT3D:String = "prepareCreateObject3d"; //未処理
		public static const PROGRESS_CREATE_OBJECT3D:String = "progressCreateObject3d";//処理中
		public static const COMPLETE_CREATE_OBJECT3D:String = "ccompleteCreateObject3d";//完了	
		
		protected var daeXML:XML; 
		protected var daeURL:String;
		
		/*MaterialsListを作成するための情報をオブジェクト化して配列管理
		{textureURL:"string", materialSymbol:"string"}*/
		protected var materialsListInfo:Array;	
		
		/*animationClipを作成するための情報をオブジェクト化して配列管理
		{animationName:"string", animationStart:num, animationStop:num }*/
		protected var animationClipsInfo:Array;	
		
		private var factoryMaterialsList:FactoryMaterialsList;
		private var _animationClips:Array;	//AnimationClip3Dを配列管理
		
		public function FactoryObject3d(autoPlay:Boolean=true, name:String=null, loop:Boolean=true) 
		{
			super(autoPlay, name, loop);
		}
		
		public function createObject3d():void
		{			
			if (!daeURL && !daeXML) trace("daeURLかdaeXMLを設定して下さい。");			
			if (!materialsListInfo) trace("texturesInfoにテクスチャの情報を入力して下さい。");			
			if (!animationClipsInfo) trace(this.name, "はアニメーションクリップが設定されていません。");
			
			loadDAE();
		}
		
		private function loadDAE():void
		{
			if (daeXML) {
				createMaterialsList()
			}else {
				if(daeURL){
					var daeURLRequest:URLRequest = new URLRequest(daeURL);
					var daeLoader:URLLoader = new URLLoader();
					daeLoader.dataFormat = URLLoaderDataFormat.TEXT;
					daeLoader.addEventListener(Event.COMPLETE,completeLoadDAE, false, 0, true);
					daeLoader.load(daeURLRequest);
				} else {
					trace("予期せぬエラーです。");
				}
			}
		}
		
		private function completeLoadDAE(event:Event):void
		{
			event.target.removeEventListener(Event.COMPLETE,completeLoadDAE);
			
			daeXML = new XML(event.target.data);
			createMaterialsList();
		}
		
		private function createMaterialsList():void
		{			
			factoryMaterialsList = new FactoryMaterialsList();
			factoryMaterialsList.addEventListener(FactoryMaterialsList.COMPLETE_MATERIALSLIST, completeMaterialsList);
			factoryMaterialsList.createMaterialsList(materialsListInfo);
		}
		
		private function completeMaterialsList(event:Event):void
		{
			event.target.removeEventListener(FactoryMaterialsList.COMPLETE_MATERIALSLIST, completeMaterialsList);
			
			this.addEventListener(FileLoadEvent.LOAD_COMPLETE, setAnimationClips, false, 0, true);
			this.load(daeXML, event.target.materialsList);
		}
		
		private function setAnimationClips(event : FileLoadEvent):void
		{
			this.removeEventListener(FileLoadEvent.LOAD_COMPLETE, setAnimationClips);
			
			if (animationClipsInfo) {		
				_animationClips = new Array();
			
				for (var i:int = 0; i < this.animationClipsInfo.length; i++) {
					_animationClips[i] = new AnimationClip3D(
														animationClipsInfo[i].animationName,
														animationClipsInfo[i].animationStart,
														animationClipsInfo[i].animationStop
														);
					this.animation.addClip(_animationClips[i]);
				}
				this.animation.play(animationClipsInfo[0].animationName, this._loop);
			}
			
			completeCreateObject3d();
		}
		
		private function completeCreateObject3d():void
		{		
			this.dispatchEvent(new Event(COMPLETE_CREATE_OBJECT3D));
		}
		
		/*以下セッターとゲッター*/
		public function get animationClips():Array
		{
			return _animationClips;
		}
		
	}//class
}//package